package ch.docuteam.darc.mdconfig;

import org.jdesktop.swingx.treetable.MutableTreeTableNode;

import ch.docuteam.darc.exceptions.MetadataElementValidatorException;
import ch.docuteam.darc.mets.structmap.NodeAbstract;
import ch.docuteam.darc.mets.structmap.NodeFolder;
import ch.docuteam.tools.translations.I18N;

/**
 * A MetadataElement validator that checks if value of an element is unique within the same level of the same subtree.
 *
 * @author christian
 *
 */
public class MetadataElementValidatorUniqueValueAmongSiblings implements MetadataElementValidator
{
	@Override
	public void check(String value, NodeAbstract context, MetadataElement metadataElement) throws MetadataElementValidatorException
	{
		//	Root has no siblings:
		if (context.isRoot())	return;

		String errorMessage = null;
		try
		{
			for (MutableTreeTableNode child: ((NodeFolder)context.getParent()).getChildren())
			{
				//	Don't compare with itself:
				if (child == context)	continue;

				if (((NodeAbstract)child).getAllDynamicMetadataValuesForName(metadataElement.getAccessorName()).contains(value))
				{
					errorMessage = I18N.translate_NoCheck("MessageValidatorUniqueValueAmongSiblings", value, I18N.translate_NoCheck(metadataElement.getId()));
					break;
				}
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			errorMessage = ex.getMessage();
		}

		if (errorMessage != null)		throw new MetadataElementValidatorException(errorMessage);
	}

}
